@@ -10,7 +10,7 @@
 #'   \item A tibble (see \code{\link[tibble]{tibble}})
 #'   \item An additional class, "cor_df"
 #'   \item A "rowname" column
-#'   \item Standardised variances (the matrix diagonal) set to missing values by
+#'   \item Standardized variances (the matrix diagonal) set to missing values by
 #'   default (\code{NA}) so they can be ignored in calculations.
 #' }
 #' 
@@ -81,66 +81,80 @@ correlate.tbl_sql <- function(x, y = NULL,
     if(!method %in% c("pearson", "spearman"))
       stop("Only pearson or spearman methods are currently supported")
     
-    df_cor <- as_cordf(
-      sparklyr::ml_corr(x, method = method)
-    )
+    df_cor <- as_cordf(sparklyr::ml_corr(x, method = method))
   }
   
   if(is.null(df_cor)){
     
     if(method != "pearson")   stop("Only 'pearson' method is currently supported")
     
-    minus_mean <- mutate_all(x, funs(. - mean(., na.rm = TRUE))) 
-    
     col_names <- colnames(x)
-    col_no <- seq_along(col_names)
     
-    cols <- map(
-      col_no,
-      ~{
-        x <- .x
-        map(col_no, ~ c(.x, x))
-      }
+    cols <- map_dfr(
+      col_names,
+      ~ tibble(
+        x = .x, 
+        y = col_names
+      ))
+    combos <- map_chr(transpose(cols),  ~ paste0(sort(c(.x$x, .x$y)), collapse = "_"))
+    cols$combos <- combos
+    unique_combos <- unique(combos)
+    
+    f_cols <- map_dfr(unique_combos, ~ head(cols[cols$combos == .x, ], 1))
+    
+    if(!all(unique(f_cols$x) == col_names)) stop("Not all variable combinations are present")
+    if(!all(unique(f_cols$y) == col_names)) stop("Not all variable combinations are present")
+    
+    f_cols <- f_cols[f_cols$x != f_cols$y, ]
+    
+    mnprod <- map(transpose(f_cols), ~expr(sum(!! sym(.x$x) * !! sym(.x$y), na.rm = TRUE)))
+    mnprod <- set_names(mnprod, f_cols$combos)
+    
+    mnsum <- map(col_names, ~expr(sum(!! sym(.x), na.rm = TRUE)))
+    mnsum <- set_names(mnsum, paste0(col_names, "_sum"))
+    
+    mntwo <- map(col_names, ~expr(sum(!! sym(.x) * !! sym(.x), na.rm = TRUE)))
+    mntwo <- set_names(mntwo, paste0(col_names, "_two"))
+    obs <- set_names(list(expr(n())), "obs")
+    db_totals <- collect(summarise(x, !!! c(mnsum, mntwo, mnprod, obs)))
+    
+    f_cols$x_sum = paste0(f_cols$x, "_sum")
+    f_cols$y_sum = paste0(f_cols$y, "_sum")
+    f_cols$x_two = paste0(f_cols$x, "_two")
+    f_cols$y_two = paste0(f_cols$y, "_two")
+    
+    l_cols <- transpose(f_cols)
+    
+    top <- map(l_cols, ~ expr((obs * !! sym(.x$combos)) - (!! sym(.x$x_sum) * !! sym(.x$y_sum))  ))
+    bottom <- map(l_cols, ~ expr((sqrt(((obs * !! sym(.x$x_two)) - (!! sym(.x$x_sum) * !! sym(.x$x_sum))) * ((obs * !! sym(.x$y_two)) - (!! sym(.x$y_sum) * !! sym(.x$y_sum)))))))
+    f_cor <- map(seq_along(top), ~expr(!! top[[.x]] / !! bottom[[.x]]))
+    f_cor <- set_names(f_cor, f_cols$combos)
+    
+    f_cors <- summarise(db_totals, !!! f_cor)
+    f_combos <- map(combos, ~ f_cors[, colnames(f_cors) == .x])
+    if("tbl_df" %in% class(f_cors)) {
+      f_combos <- map(f_combos, ~ ifelse(nrow(.x) > 0, .x[1,], 0)[[1]])
+    } else {
+      f_combos <- map(f_combos, ~ ifelse(!is.null(nrow(.x)), NA, .x))
+    }
+    f_combos <- map_dbl(f_combos, ~ifelse(is.null(.x), NA, .x))
+    
+    cor_tbl <- cols
+    cor_tbl$cor <- f_combos
+    cor_tbl$xn <- map_int(
+      cor_tbl$x,
+      ~which(.x == col_names)
     )
-    cols <- flatten(cols)
-    cols <- map(cols, sort)
-    
-    dups <- map_lgl(cols, ~all.equal(.x[1], .x[2]) != TRUE)
-    
-    cols <- cols[dups]
-    
-    cols_names <-map(cols, paste0, collapse = ",")
-    
-    unique_cols <- unique(cols_names)
-    
-    combos <- map(
-      unique_cols,
-      ~ cols[cols_names == .x][[1]]
+    cor_tbl$yn <- map_int(
+      cor_tbl$y,
+      ~which(.x == col_names)
     )
-    
-    f_cor <- map(
-      combos,
-      ~{
-        l <- col_names[.x[[1]]]
-        r <- col_names[.x[[2]]]
-        map2(l, r, tidyeval_cor)
-      }
-    )
-    f_cor <- flatten(f_cor)
-    f_cor <- set_names(f_cor, unique_cols)
-    
-    cors <- summarise(minus_mean, !!! f_cor)
-    cors <- collect(cors)
-    
     cors_matrix <- matrix(
-      ncol = length(col_no), 
-      nrow = length(col_no)
+      ncol = length(col_names), 
+      nrow = length(col_names)
     )
-    
-    for(i in seq_along(cors)){
-      loc <- combos[unique_cols == names(cors[i])][[1]]
-      cors_matrix[loc[1], loc[2]] <- cors[i][[1]]
-      cors_matrix[loc[2], loc[1]] <- cors[i][[1]]
+    for(i in seq_along(combos)){
+      cors_matrix[cor_tbl$xn[[i]], cor_tbl$yn[[i]]] <- cor_tbl$cor[[i]]
     }
     colnames(cors_matrix) <- col_names
     df_cor <- as_cordf(cors_matrix)
@@ -154,17 +168,4 @@ correlate.tbl_sql <- function(x, y = NULL,
   }
   df_cor
 }
-
-tidyeval_cor <- function(x, y) {
-  x <- sym(enexpr(x))
-  y <- sym(enexpr(y))
-  expr(
-    sum(!! x * !! y , na.rm = TRUE) /
-      sqrt(
-        sum(!! x * !! x, na.rm = TRUE) *
-          sum(!! y * !! y, na.rm = TRUE)
-      )
-  )
-}
-
 utils::globalVariables(c("rowname"))
