@@ -9,14 +9,13 @@ test_that("Converts to proper structure", {
     nrow(stretch(d)),
     nrow(d) * nrow(d)
   )
-  expect_output(
-    str(stretch(d)),
-    "16 obs. of  3 variables:"
-  )
   expect_equal(
     colnames(stretch(d)),
     c("x", "y", "r")
   )
+  verify_output("stretch.txt",
+    str(stretch(d))
+  )
 })
 
 test_that("na.rm", {
